@@ -1,10 +1,10 @@
-function preresolve ( ip, domain, qtype )
-	print ("prequery handler called for: ", ip, domain, qtype)
-
+function preresolve ( remoteip, domain, qtype )
+	print ("prequery handler called for: ", remoteip, getlocaladdress(), domain, qtype)
+	pdnslog("a test message.. received query from "..remoteip.." on "..getlocaladdress());
 	if domain == "www.powerdns.org." 
 	then
 		ret={}
-		ret[1]= {qtype=pdns.A, content="85.17.219.141", ttl=86400}
+		ret[1]= {qtype=pdns.A, content="85.17.220.215", ttl=86400}
 		print "dealing!"
 		return 0, ret
 	elseif domain == "www.baddomain.com."
@@ -14,23 +14,23 @@ function preresolve ( ip, domain, qtype )
 	elseif domain == "echo."
 	then
 		print "dealing with echo!"
-		return 0, {{qtype=pdns.A, content=ip}}
+		return 0, {{qtype=pdns.A, content=remoteip}}
 	elseif domain == "echo6."
 	then
 		print "dealing with echo6!"
-		return 0, {{qtype=pdns.AAAA, content=ip}}
+		return 0, {{qtype=pdns.AAAA, content=remoteip}}
 	else
 		print "not dealing!"
 		return -1, {}
 	end
 end
 
-function nxdomain ( ip, domain, qtype )
-	print ("nxhandler called for: ", ip, domain, qtype, pdns.AAAA)
+function nxdomain ( remoteip, domain, qtype )
+	print ("nxhandler called for: ", remoteip, getlocaladdress(), domain, qtype, pdns.AAAA)
 	if qtype ~= pdns.A then return -1, {} end  --  only A records
 	if not string.find(domain, "^www%.") then return -1, {} end  -- only things that start with www.
 	
-	if matchnetmask(ip, {"127.0.0.1/32", "10.1.0.0/16"}) 
+	if matchnetmask(remoteip, {"127.0.0.1/32", "10.1.0.0/16"}) 
 	then 
 		print "dealing"
 		ret={}
